//////////////////////////////////////////////////////////////////////////////
//
// AutoPitLot v 2.0
// Obsuga magnetometru MMX312x na magistrali I2C
//
// (c) Pit Lab
// http://www.pitlab.pl
//////////////////////////////////////////////////////////////////////////////
#include "LPC1768.h"
#include "mmc312x.h"
#include "errors.h"
#include "sys_def.h"
#include "config_fram.h"
#include <math.h>
#include "i2c_async.h"


// Obsuga magnetometru wymaga wykonania kilku czynnoci rozoonych w czasie
// 1) Wystartowanie konwersji trwajcej 5ms (slot czasowy 0)
// 2) Wystartowanie odczytu wykonanego pomiaru. Odczyt wykonuje si w tle w procedurze przerwania I2C i 
//    trwa ok. 4ms (slot 11)
// 3) Przepisania odczytanych w przerwaniu danych do zmiennych pomiarowych

// Zmienne kalibracyjne
// 1) Zmienna FAH_MAG_MMC_CORR - korekcja zera mostka magnetometru kompensuje bdy nierwnomiernoci rozkadu 
//   pola w samym mostku. Uzyskuje si j przez odjecie od siebie wynikw pomiaru uzyskanych dla obu kierunkw
//   wstepnego namagnesownia struktury pomiarowej poleceniami SET i RESET. Zmiena = pomiar(SET) - pomiar(RESET)
// 2) Zmienna FAH_MAG_MMC_GAIN koryguje zaburzenie pola spododowane obecnoci ferromegnetykw obok czujnika
//   Powoduj one spaszczenie lub wyciagnicie pola w danej osi. Przemnoenie sygnau przez wspczynnik
//   daje jego normalizacj.
// 3) Zmiena FAH_MAG_MMC_OFST koryguje pooenie znormalizowanej wartoci pola magnetycznego, tak aby maksyma 
//   sygnau leay w rnych odlegociach od zera.  Zmienna = (Max + -Min)/2


////////////////////////////////////////////////////////////////////////////////
// Budzi ukad i startuje konwersj magnetometru MMC3120MQ
// Dane powinny pojawi si 5ms po starcie konwersji
// Parametry: 
// [i] chChan - kana magistrali I2C 1 lub 2
// Zwraca: nic
// Czas zajcia magistrali I2C: 1,7ms przy zegarze 50kHz
////////////////////////////////////////////////////////////////////////////////
unsigned char StartMagnetometerMMC(unsigned char chChan)
{
    unsigned char chErr;

    switch(chChan)
    {
    case 0: //I2C0
        chData0[0] = 0x01;   //wake up
        chErr = StartI2C0(0x60, 0x00, chData0, 1, 0);   //wylij polecenie obudzenia czci pomiarowej
        break;
    case 1: //I2C1
        chData1[0] = 0x01;   //wake up
        chErr = StartI2C1(0x60, 0x00, chData1, 1, 0);   //wylij polecenie obudzenia czci pomiarowej
        break;
    case 2: //I2C2
        chData2[0] = 0x01;   //wake up
        chErr = StartI2C2(0x60, 0x00, chData2, 1, 0);   //wylij polecenie obudzenia czci pomiarowej
        break;
    }
    return chErr;
}


////////////////////////////////////////////////////////////////////////////////
// Inicjuje odczyt danych z magnetometru MMC3120MQ
// Parametry: 
// [i] chChan - kana magistrali I2C 1 lub 2
// Zwraca: nic
// Czas zajcia magistrali I2C: 4,0ms przy zegarze 50kHz, 290us przy 400kHz
////////////////////////////////////////////////////////////////////////////////
unsigned char ReadMagnetometerMMC(unsigned char chChan)
{
    unsigned char chErr;

    switch(chChan)
    {
    case 0: //I2C0
        chErr = StartI2C0(0x60, 0x00, chData0, 0, 7);   //wylij polecenie zapisu adresu 0 i odczytu 7 bajtw danych
        break;
    case 1: //I2C1
        chErr = StartI2C1(0x60, 0x00, chData1, 0, 7);   //wylij polecenie zapisu adresu 0 i odczytu
        break;
    case 2: //I2C2
        chErr = StartI2C2(0x60, 0x00, chData2, 0, 7);   //wylij polecenie zapisu adresu 0 i odczytu
        break;
    }
    return chErr;
}


////////////////////////////////////////////////////////////////////////////////
// Wykonuje operacj SET lub RESET magnetometru przemagnesowywujc struktur czujnika
// Parametry: 
// [i] chSet - 2=SET, 4=RESET
// [i] chChan - kana magistrali I2C 1 lub 2
// Zwraca: nic
////////////////////////////////////////////////////////////////////////////////
unsigned char ReSetMagnetometerMMC(unsigned char chChan, unsigned char chSet)
{
    unsigned char chErr;

    switch(chChan)
    {
    case 1: //I2C1
        chData1[0] = chSet;  //polecenie SET lub RESET
        chErr = StartI2C1(0x60, 0x00, chData1, 2, 0);   //wylij polecenie 
        break;
    case 2: //I2C2
        chData2[0] = chSet;  //polecenie SET lub RESET
        chErr = StartI2C2(0x60, 0x00, chData2, 2, 0);   //wylij polecenie 
        break;
    }
    return chErr;
}


////////////////////////////////////////////////////////////////////////////////
// Analizuje dane i liczy wartoci natenia pola magnetycznego we wszystkich 3 osiach
// Na razie kalibracja jest wpieta na sztywno w kod. Docelowo trzeba przenie do FRAM
// Parametry: 
// [o] *fValXYZ - skdowa wektora magnetycznago dla kadej osi
// Zwraca: kod bdu
////////////////////////////////////////////////////////////////////////////////
//unsigned char CalculateMagnetometerMMC0(float *fValX, float *fValY, float *fValZ)
unsigned char CalculateMagnetometerMMC0(void)
{
    float fTemp;
    unsigned char x, y;

    if (chData0[0] != 0)
        return ERR_NO_MMC_MEASUREM;

    if ((!chData0[1] && !chData0[2]) | (!chData0[3] && !chData0[4]) | (!chData0[5] && !chData0[6]))
        return ERR_INVALID_DATA;

    for (x=0; x<3; x++)
    {
        y = 2*x;
        fVectMagnMMC0[x] = 0x100 * chData0[y+1] + chData0[y+2];
    }

    //wykryj rodzaj ukadu magnetometru
    fTemp = (fVectMagnMMC0[0] + fVectMagnMMC0[1] + fVectMagnMMC0[2])/3;    
    if (fTemp > 3072)
    {
        //obsugujemy ukad MMC314x i trzeba odj offset 4096
        for (x=0; x<3; x++)
            fVectMagnMMC0[x] -= 4096;
    }
    else
    {
        //obsugujemy ukad MMC312x i trzeba odj offset 2048
        for (x=0; x<3; x++)
            fVectMagnMMC0[x] -= 2048;
    }

    //jeeli trwa pomiar korekcji magnetometru na magistrali I2C1 to zwr surowe dane
    if (sInitFlag & FIND_CORR0)
        return ERR_OK;

    for (x=0; x<3; x++)
        fVectMagnMMC0[x] -= fCorrMagnMMC0[x];

    //jeeli trwa kalibracja to uwzgldniaj tylko wspczynnik korekcyjny
    if (sInitFlag & CALIB_MAG0)
        return ERR_OK;

    //normalny pomiar z uwzglednieniem wszystkich wsplczynikw korekcyjnych
    for (x=0; x<3; x++)
        fVectMagnMMC0[x] = fVectMagnMMC0[x] * fGainMagnMMC0[x] - fOfstMagnMMC0[x];     
    return ERR_OK;
}


////////////////////////////////////////////////////////////////////////////////
// Analizuje dane i liczy wartoci natenia pola magnetycznego we wszystkich 3 osiach
// Na razie kalibracja jest wpieta na sztywno w kod. Docelowo trzeba przenie do FRAM
// Parametry: 
// [o] *fValXYZ - skadowa wektora magnetycznago dla kadej osi
// Zwraca: kod bdu
////////////////////////////////////////////////////////////////////////////////
unsigned char CalculateMagnetometerMMC1(void)
{
    float fTemp;
    unsigned char x, y;

    if (chData1[0] != 0)
        return ERR_NO_MMC_MEASUREM;

    if ((!chData1[1] && !chData1[2]) | (!chData1[3] && !chData1[4]) | (!chData1[5] && !chData1[6]))
        return ERR_INVALID_DATA;

    for (x=0; x<3; x++)
    {
        y = 2*x;
        fVectMagnMMC1[x] = 0x100 * chData1[y+1] + chData1[y+2];
    }

    //wykryj rodzaj ukadu magnetometru
    fTemp = (fVectMagnMMC1[0] + fVectMagnMMC1[1] + fVectMagnMMC1[2])/3;
    if (fTemp > 3072)
    {
        //obsugujemy ukad MMC314x i trzeba odj offset 4096
        for (x=0; x<3; x++)
            fVectMagnMMC1[x] -= 4096;
    }
    else
    {
        //obsugujemy ukad MMC312x i trzeba odj offset 2048
        for (x=0; x<3; x++)
            fVectMagnMMC1[x] -= 2048;
    }

    //jeeli trwa pomiar korekcji magnetometru na magistrali I2C1 to zwr surowe dane
    if (sInitFlag & FIND_CORR1)
        return ERR_OK;

    for (x=0; x<3; x++)
        fVectMagnMMC1[x]-= fCorrMagnMMC1[x];

    //jeeli trwa kalibracja to uwzgldniaj tylko wspczynnik korekcyjny
    if (sInitFlag & CALIB_MAG1)
        return ERR_OK;

    //normalny pomiar z uwzglednieniem wszystkich wsplczynikw korekcyjnych
    for (x=0; x<3; x++)
        fVectMagnMMC1[x] = fVectMagnMMC1[x] * fGainMagnMMC1[x] - fOfstMagnMMC1[x]; 

    return ERR_OK;
}


unsigned char CalculateMagnetometerMMC2(void)
{
    float fTemp;
    unsigned char x, y;

    if (chData2[0] != 0)
        return ERR_NO_MMC_MEASUREM;

    if ((!chData2[1] && !chData2[2]) | (!chData2[3] && !chData2[4]) | (!chData2[5] && !chData2[6]))
        return ERR_INVALID_DATA;

    for (x=0; x<3; x++)
    {
        y = 2*x;
        fVectMagnMMC2[x] = 0x100 * chData2[y+1] + chData2[y+2];
    }

    //wykryj rodzaj ukadu magnetometru
    fTemp = (fVectMagnMMC2[0] + fVectMagnMMC2[1] + fVectMagnMMC2[2])/3;
    if (fTemp > 3072)
    {
        //obsugujemy ukad MMC314x i trzeba odj offset 4096
        for (x=0; x<3; x++)
            fVectMagnMMC2[x] -= 4096;
    }
    else
    {
        //obsugujemy ukad MMC312x i trzeba odj offset 2048
        for (x=0; x<3; x++)
            fVectMagnMMC2[x] -= 2048;
    }

    //jeeli trwa pomiar korekcji magnetometru na magistrali I2C1 to zwr surowe dane
    if (sInitFlag & FIND_CORR2)
        return ERR_OK;

    for (x=0; x<3; x++)
        fVectMagnMMC2[x] = fRawMag2[x] - fCorrMagnMMC2[x];

    //jeeli trwa kalibracja to uwzgldniaj tylko wspczynnik korekcyjny
    if (sInitFlag & CALIB_MAG2)
        return ERR_OK;

    //normalny pomiar z uwzglednieniem wszystkich wsplczynikw korekcyjnych
    for (x=0; x<3; x++)
        fVectMagnMMC2[x] = fVectMagnMMC2[x] * fGainMagnMMC2[x] - fOfstMagnMMC2[x];     
    return ERR_OK;
}


////////////////////////////////////////////////////////////////////////////////
// Funkcja kapsukujca funkcj obliczeniow. Pozwala na uycie na zewntrz bez potrzeby publikowania danych
// Parametry: 
// [i] chChan - kana magistrali I2C 0, 1 lub 2
// Zwraca: kod bdu
////////////////////////////////////////////////////////////////////////////////
unsigned char MeasureMagnetometerMMC(unsigned char chChan)
{
    unsigned char x, chErr;

    switch(chChan)
    {
    case 0: //I2C0
        //chErr = CalculateMagnetometerMMC0(&fVectMagnMMC0[0], &fVectMagnMMC0[1], &fVectMagnMMC0[2]);
        chErr = CalculateMagnetometerMMC0();
        /*if (chCalibMag & CAL_ACTIVE_MAG1)   //jeeli trwa kalibracja to znajd wartoci maksymalne
        {
          for (x=0; x<3; x++)
          {
            if (fVectMagnMMC0[x] > fMaxValMagn0[x])
            {
              fMaxValMagn0[x] = fVectMagnMMC0[x];
              BeepMax();
            }
            if (fVectMagnMMC0[x] < fMinValMagn0[x])
            {
              fMinValMagn0[x] = fVectMagnMMC0[x];
              BeepMin();
            }
          }
        }*/
        break;

    case 1: //I2C1
        //chErr = CalculateMagnetometerMMC1(&fVectMagnMMC1[0], &fVectMagnMMC1[1], &fVectMagnMMC1[2]);
        chErr = CalculateMagnetometerMMC1();
        if (sInitFlag & CALIB_MAG1)     //jeeli trwa kalibracja to znajd wartoci maksymalne
        {
          for (x=0; x<3; x++)
          {
            if (fVectMagnMMC1[x] > fMaxValMagn1[x])
            {
              fMaxValMagn1[x] = fVectMagnMMC1[x];
              BeepMax();
            }
            if (fVectMagnMMC1[x] < fMinValMagn1[x])
            {
              fMinValMagn1[x] = fVectMagnMMC1[x];
              BeepMin();
            }
          }
        }
        break;

    case 2: //I2C2
        chErr = CalculateMagnetometerMMC2();
        if (sInitFlag & CALIB_MAG2)     //jeeli trwa kalibracja to znajd wartoci maksymalne
        {
          for (x=0; x<3; x++)
          {
            if (fVectMagnMMC2[x] > fMaxValMagn2[x])
            {
              fMaxValMagn2[x] = fVectMagnMMC2[x];
              BeepMax();
            }
            if (fVectMagnMMC2[x] < fMinValMagn2[x])
            {
              fMinValMagn2[x] = fVectMagnMMC2[x];
              BeepMin();
            }
          }
        }
        break;
    }
    return chErr;
}


////////////////////////////////////////////////////////////////////////////////
// Inicjuje zmienne konfiguracyjne
// Parametry: 
// Zwraca: kod bdu
////////////////////////////////////////////////////////////////////////////////
unsigned char InitMagnetometerMMC(void)
{
    unsigned char x, chErr = ERR_OK;   

    //odczytaj wartoci kalibracyjne magnetometru
    for (x=0; x<3; x++)
    {
        //offset zera pocztkowego magnetometru
        chErr |= FramDataReadFloatValid((unsigned short)(FAH_MAG_MMC1_CORRX+x*4), &fCorrMagnMMC1[x], VALM_CORR_MAGN, VALP_CORR_MAGN, VALD_CORR_MAGN, ERR_MAGN_CAL_VAL);
        chErr |= FramDataReadFloatValid((unsigned short)(FAH_MAG_MMC2_CORRX+x*4), &fCorrMagnMMC2[x], VALM_CORR_MAGN, VALP_CORR_MAGN, VALD_CORR_MAGN, ERR_MAGN_CAL_VAL);

        //wzmocnienie wskaza magnetometru
        chErr |= FramDataReadFloatValid((unsigned short)FAH_MAG_MMC1_GAINX+x*4, &fGainMagnMMC1[x], VALM_GAIN_MAGN, VALP_GAIN_MAGN, VALD_GAIN_MAGN, ERR_MAGN_CAL_VAL);
        chErr |= FramDataReadFloatValid((unsigned short)FAH_MAG_MMC2_GAINX+x*4, &fGainMagnMMC2[x], VALM_GAIN_MAGN, VALP_GAIN_MAGN, VALD_GAIN_MAGN, ERR_MAGN_CAL_VAL);
        
        //offset kompensacji hard/soft iron magnetometru
        chErr |= FramDataReadFloatValid((unsigned short)FAH_MAG_MMC1_OFSTX+x*4, &fOfstMagnMMC1[x], VALM_OFST_MAGN, VALP_OFST_MAGN, VALD_OFST_MAGN, ERR_MAGN_CAL_VAL);
        chErr |= FramDataReadFloatValid((unsigned short)FAH_MAG_MMC2_OFSTX+x*4, &fOfstMagnMMC2[x], VALM_OFST_MAGN, VALP_OFST_MAGN, VALD_OFST_MAGN, ERR_MAGN_CAL_VAL);

        //zaprogramuj wykonanie 3 cykli SET-RESET
        SetAsyncI2C_Cmd(I2CMD_SET_MAGN);
        SetAsyncI2C_Cmd(I2CMD_RES_MAGN);
    }

    if (!chErr)
        sInitFlag |= IDONE_MAG; //zakonczono kalibracj magnetometru
    return chErr;
}


////////////////////////////////////////////////////////////////////////////////
// Rozpoczyna i koczy kolejne etapy procedury kalibracyjnej magetometrw na obu magistralach
// Parametry: 
// [i] chEtap - etapy kalibracji: 
//  0 = zerowanie wspczynnikw korekcji na magistrali I2C1
//  1 = zerowanie wspczynnikw korekcji na magistrali I2C2
//  2 = ponowne wczytanie zewntrznie zapisanych zmiennych korekcji na magistrali I2C1
//  3 = ponowne wczytanie zewntrznie zapisanych zmiennych korekcji na magistrali I2C2
//  5 = ustawienie flagi kalibracji magnetometru na magistrali I2C1
//  6 = ustawienie flagi kalibracji magnetometru na magistrali I2C2
//  7 = kasowanie flagi kalibracji i obliczenie wspczynnikw kalibracji magnetometru na obu magistralach
// Zwraca: kod bdu
////////////////////////////////////////////////////////////////////////////////
unsigned char CalibrateMagnMMC(unsigned char chEtap)
{
    unsigned char x, chErr = ERR_OK;
    float fVect3D;
    
    switch (chEtap)
    {
    case 0: //zeruj korekcj na kanale I2C1
        sInitFlag |= FIND_CORR1;
        for (x=0; x<3; x++)
            fCorrMagnMMC1[x] = 0.0;
        break;

    case 1: //zeruj korekcj na kanale I2C2
        sInitFlag |= FIND_CORR2;
        for (x=0; x<3; x++)
            fCorrMagnMMC2[x] = 0.0;
        break;

    case 2:  //przeaduj konfiguracj  na kanale I2C1
        sInitFlag &= ~FIND_CORR1;
        for (x=0; x<3; x++)
        {
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC1_CORRX+x*4, &fCorrMagnMMC1[x], VALM_CORR_MAGN, VALP_CORR_MAGN, VALD_CORR_MAGN, ERR_MAGN_CAL_VAL);
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC1_GAINX+x*4, &fGainMagnMMC1[x], VALM_GAIN_MAGN, VALP_GAIN_MAGN, VALD_GAIN_MAGN, ERR_MAGN_CAL_VAL);           
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC1_OFSTX+x*4, &fOfstMagnMMC1[x], VALM_OFST_MAGN, VALP_OFST_MAGN, VALD_OFST_MAGN, ERR_MAGN_CAL_VAL);            
        }
        break;

    case 3: //przeaduj konfiguracj  na kanale I2C2
        sInitFlag &= ~FIND_CORR2;
        for (x=0; x<3; x++)
        {
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC2_CORRX+x*4, &fCorrMagnMMC2[x], VALM_CORR_MAGN, VALP_CORR_MAGN, VALD_CORR_MAGN, ERR_MAGN_CAL_VAL);
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC2_GAINX+x*4, &fGainMagnMMC2[x], VALM_GAIN_MAGN, VALP_GAIN_MAGN, VALD_GAIN_MAGN, ERR_MAGN_CAL_VAL);
            chErr |= FramDataReadFloatValid(FAH_MAG_MMC2_OFSTX+x*4, &fOfstMagnMMC2[x], VALM_OFST_MAGN, VALP_OFST_MAGN, VALD_OFST_MAGN, ERR_MAGN_CAL_VAL);
        }
        break;

    case 5: //ustaw flag kalibracji magnetometru na magistrali I2C1
        sInitFlag |= CALIB_MAG1;
        for (x=0; x<3; x++)
        {
            fMinValMagn1[x] = fVectMagnMMC1[x];
            fMaxValMagn1[x] = fVectMagnMMC1[x];
        }
        break;

    case 6: //ustaw flag kalibracji magnetometru na magistrali I2C2
        sInitFlag |= CALIB_MAG2;
         for (x=0; x<3; x++)
        {
            fMinValMagn2[x] = fVectMagnMMC2[x];
            fMaxValMagn2[x] = fVectMagnMMC2[x];
        }
        break;

    case 7: //wykonaj kalibraj i kasuj flagi kalibracji magnetometru na magistralach I2C1 i I2C2
        if (sInitFlag & CALIB_MAG1)
        {
            //oblicz znormalizowan warto wektora 3D ekstremw pola magnetycznego
            fVect3D = sqrtf((fMaxValMagn1[0] - fMinValMagn1[0]) * (fMaxValMagn1[0] - fMinValMagn1[0])+
                            (fMaxValMagn1[1] - fMinValMagn1[1]) * (fMaxValMagn1[1] - fMinValMagn1[1])+
                            (fMaxValMagn1[2] - fMinValMagn1[2]) * (fMaxValMagn1[2] - fMinValMagn1[2]));

            //oblicz a nastpnie zapisz gain i ofset dla czujnika na magistrali I2C1
            for (x=0; x<3; x++)
            {
                fGainMagnMMC1[x] = fVect3D / (fMaxValMagn1[x] - fMinValMagn1[x]);
                fOfstMagnMMC1[x] = (fMaxValMagn1[x] + fMinValMagn1[x]) / 2 * fGainMagnMMC1[x];
                FramDataWriteFloat(FAH_MAG_MMC1_GAINX+x*4, fGainMagnMMC1[x]);
                FramDataWriteFloat(FAH_MAG_MMC1_OFSTX+x*4, fOfstMagnMMC1[x]);
            }
            sInitFlag &= ~CALIB_MAG1;
        }
        if (sInitFlag & CALIB_MAG2)
        {
            //oblicz znormalizowan warto wektora 3D ekstremw pola magnetycznego
            fVect3D = sqrtf((fMaxValMagn2[0] - fMinValMagn2[0]) * (fMaxValMagn2[0] - fMinValMagn2[0])+
                            (fMaxValMagn2[1] - fMinValMagn2[1]) * (fMaxValMagn2[1] - fMinValMagn2[1])+
                            (fMaxValMagn2[2] - fMinValMagn2[2]) * (fMaxValMagn2[2] - fMinValMagn2[2]));

            //oblicz a nastpnie zapisz gain i ofset dla czujnika na magistrali I2C2
            for (x=0; x<3; x++)
            {
                //fGainMagnMMC2[x] = (float)MAX_VAL / (fMaxValMagn2[x] - fMinValMagn2[x]);
                fGainMagnMMC2[x] = fVect3D / (fMaxValMagn2[x] - fMinValMagn2[x]);
                fOfstMagnMMC2[x] = ((fMaxValMagn2[x] + fMinValMagn2[x]) / 2) * fGainMagnMMC2[x];
                FramDataWriteFloat(FAH_MAG_MMC2_GAINX+x*4, fGainMagnMMC2[x]);
                FramDataWriteFloat(FAH_MAG_MMC2_OFSTX+x*4, fOfstMagnMMC2[x]);
            }
            sInitFlag &= ~CALIB_MAG2;
        }
        break;

    case 8: //zakocz kalibracj i korekcj na wszystkich kanaach
        sInitFlag &= ~(FIND_CORR1 | FIND_CORR2 | CALIB_MAG1 | CALIB_MAG2);
        break;

    case 9: //inicjalizuj dane do wartoci domylnych na I2C1
        for (x=0; x<3; x++)
        {
            fGainMagnMMC1[x] = (float)1.0;
            fOfstMagnMMC1[x] = (float)0.0;
            FramDataWriteFloat(FAH_MAG_MMC1_GAINX+x*4, fGainMagnMMC1[x]);
            FramDataWriteFloat(FAH_MAG_MMC1_OFSTX+x*4, fOfstMagnMMC1[x]);
        }
        break;

    case 10: //inicjalizuj dane do wartoci domylnych na I2C2
        for (x=0; x<3; x++)
        {
            fGainMagnMMC2[x] = (float)1.0;
            fOfstMagnMMC2[x] = (float)0.0;
            FramDataWriteFloat(FAH_MAG_MMC2_GAINX+x*4, fGainMagnMMC2[x]);
            FramDataWriteFloat(FAH_MAG_MMC2_OFSTX+x*4, fOfstMagnMMC2[x]);
        }
        break;

    default: 
        chErr = ERR_INVALID_DATA;
    }
    return chErr;
}




/*
    //metoda najmniejszych kwadratw
	int i;
	float x, y;
	float Sx, Sy, Sx2, Sxy, Sy2;
	float Wyzn;	
	
	//policz sumy porednie
	Sx  = 0;
	Sx2 = 0;
	Sy  = 0;
	Sxy = 0;
	Sy2 = 0;

	for (i=0; i<size; i++)
	{
		x = array[i].fTmp;
		y = array[i].fWys;	
		
		Sx  += y;
		Sx2 += y*y;
		Sy  += x;
		Sxy += x*y;
		Sy2 += x*x;
	}
	
	Wyzn = (float)(Sx2*size - Sx*Sx);	//licze wyznacznik macierzy SX
	if (Wyzn != 0)
		Wyzn = 1/Wyzn;			//i jego odwrotnosc
	else
	{
		//przypadek dzielenia przez zero
		Wsp.a = Wsp.b = 0;
		return Wsp;
	}

	Wsp.a = Wyzn * ( Sxy * size - Sy * Sx );
	Wsp.b = Wyzn * ( Sx2 * Sy - Sx * Sxy ); 

*/